package app;

import static java.lang.Math.PI;
import static java.lang.Math.atan;
import static java.lang.Math.log10;
import static java.lang.Math.pow;
import gui.LinkLabel;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Contiene le costanti usate dai pannelli e alcuni metodi di utilit
 * 
 * @author Devil_Dj
 * 
 */
public final class UTIL {

	private UTIL() {
	} // non  necessario creare istanze di questa classe

	// Limitano superiormente e inferiormente il valore del modulo nel grafico
	public static final int M_MIN_DEFAULT = -100;
	public static final int M_MAX_DEFAULT = 50;
	public static final int MINIMO_M_MAX = 20;
	public static final int MASSIMO_M_MIN = -20;

	// Limitano superiormente e inferiormente il valore della fase nel grafico
	public static final int F_MIN_DEFAULT = -400;
	public static final int F_MAX_DEFAULT = 180;
	public static final int MINIMA_F_MAX = 20;
	public static final int MASSIMA_F_MIN = -20;

	public static final int LOGWMIN_DEFAULT = -1;
	public static final int LOGWMAX_DEFAULT = 3;
	
	public static final int DISTANZA_NUMERETTI_ASSE_ORDINATE = 30;

	public static Color PANEL_BACKGR = Color.white;
	public static Color ASSE_PRINCIPALE = Color.black;
	public static Color ASSE_SECONDARIO = new Color(225, 225, 225);
	public static Color COLORE_FUNZIONE_REALE = Color.blue;
	public static Color COLORE_FUNZIONE_ASINT = Color.green;
	public static boolean LIVELLO_ASINT_SOTTO = true;
	public static boolean DISEGNA_GRAFICO_REALE = true;
	public static boolean DISEGNA_GRAFICO_ASINTOTICO = true;
	
	//il font delle label nella finestra del grafico
	public static final Font LABEL_FONT = new Font("Arial", 0, 12);
	public static final Font SCALA_FONT = new Font("Arial", 0, 10);
	
	//posizione iniziale della finestra
	public static final int X_DEF = 220;
	public static final int Y_DEF = 90;
	public static final int W_DEF = 800;
	public static final int H_DEF = 600;
	
	//bordi tra i grafici
	public static final int UPPER_BORDER = 80;
	public static final int MIDDLE_BORDER = 10;
	public static final int LOWER_BORDER = 30;
	public static final int LEFT_BORDER = 35;
	public static final int RIGHT_BORDER = 15;
	public static final int W_LABEL_VERT = 35;
	public static final int H_LABEL_ORIZ = 20;
	
	//dimensioni minime della finestra
	public static final int MIN_W = 300;
	public static final int MIN_H = 300;
	public static boolean MESSAGGIO_NO_ASINT_PER_FCC_VISUALIZZATO = false;
	

	/**
	 * Trova l'esponente della notazione scientifica del numero n. Vale a dire
	 * che se n=354, nella notazione scientifica sar n=3,54 * 10^2 Questo
	 * metodo quindi restituisce 2
	 * 
	 * @param n
	 * @return l'esponente della notazione scientifica del numero n.
	 */
	public static int trovaEsponenteNS(double n) {
		n=Math.abs(n);
		int cont = 0;
		if (n > 1) {
			while (n >= 10) {
				n = n / 10;
				cont++;
			}
		} else {
			while (n < 1) {
				n = n * 10;
				cont--;
			}
		}
		return cont;
	}

	/**
	 * Taglia i decimali di un double. Esempio: se d=3,4657 e precision=2 il
	 * risultato sar una stringa con 3,46. Da notare che il numero viene
	 * semplicemente tagliato, quindi non viene fatta nessuna approssimazione.
	 * 
	 * @param d
	 *            il numero da formattare
	 * @param precision
	 *            il numero di cifre dopo la virgola
	 * @return una stringa con il numero formattato.
	 */
	public static String formatDouble(double d, int precision) {
		if (precision < 0)
			throw new IllegalArgumentException("Deve essere: precision >= 0");
		
		if(precision == 0)
			return ((int)d)+"";
		
		/*
		 * esempio d=12,3456, precision=2
		 * n1 = d*10^2 ->1234,56
		 * n2 = (int) n1 -> 1234
		 * n3 = n2 + round -> 1235
		 * (double) n4 = n3/10^2 ->12,35
		 * 
		 */
		double n1 = d*pow(10, precision);
		int n2 = (int) n1;
		
		//calcolo di round
		boolean round = false;
		for(int i=1; i<10; i++){
			int c = getCifraDecimale(n1, i);
			if(c==5) continue;
			if(c>5) round = true;
			else round = false;
			break;
		}
		
		int n3 = round ? n2+1 : n2;
		double n4 = n3/pow(10, precision);
		return ""+n4;
	}
	
	/**
	 * ritorna la cifra i-esima dopo la virgola di n
	 * @param n
	 * @param i
	 * @return
	 */
	private static int getCifraDecimale(double n, int i){
		if(!(i>0))
			throw new IllegalArgumentException("deve essere i>0");
		
		/*
		 * esempio di cosa succede per n= 12,3 e i = 2
		 * 
		 * n1 = 12,345
		 * n2 = n1 * 10^2   (1234,5)
		 * n3 = (int) n2  (1234)
		 * n4 = n3%10     (4)
		 */
		double n2 = n*pow(10, i);
		int n3 = (int)n2;
		int n4 = n3%10;
		return n4;
	}

	/**
	 * ritorna il valore in dB di n
	 * 
	 * @param n
	 * @return
	 */
	public static double DB(double n) {
		return 20 * log10(n);
	}
	
	/**
	 * riceve un angolo in radianti e ritorna un angolo in gradi.
	 * @param rad
	 * @return
	 */
	public static double radToGrad(double rad){
		return (double)180*rad/PI;
	}
	
	/**
	 * ritorna la fase in gradi di un numero complesso
	 * 
	 * @param Re
	 *            la parte reale del numero complesso
	 * @param Im
	 *            la parte immaginaria del numero complesso
	 * @return la fase in gradi per il numero complesso specificato
	 */
	public static double fase(double Re, double Im){
		if(Re>=0){
			return radToGrad(atan(Im/Re));
		}else{
			return radToGrad( atan(Im/Re) + (Im>0 ? PI : -PI) );
		}
	}
	
	public static void aggiungiNumerettoV(JPanel p, String val, int x, int y){
		JLabel l = new JLabel(val);
		int h = 18;
		l.setBounds(x, y-h/2, W_LABEL_VERT-5, h);
		l.setHorizontalAlignment(JLabel.RIGHT);
		l.setVerticalAlignment(JLabel.CENTER);
		l.setFont(SCALA_FONT);
		p.add(l);
	}
	
	public static void aggiungiNumerettoO(JPanel p, String val, int x, int y){
		JLabel l = new JLabel(val);
		int h = 18;
		l.setBounds(x, y, W_LABEL_VERT, h);
		l.setVerticalAlignment(JLabel.CENTER);
		l.setFont(SCALA_FONT);
		p.add(l);
	}
	
	/**
	 * Calcola il punto tale da porre la finestra al centro dello schermo
	 * @param frameSize la dimensione del frame da posizionare
	 * @return il punto dove posizionare la finestra (x=0, y=0)
	 */
	public static Point centerOnScreen(Dimension frameSize){
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (d.width - frameSize.width)/2;
		int y = (d.height - frameSize.height)/2;
		return new Point(x, y);
	}
	
	
	
	/**
	 * ritora una ImageIcon scalata e adattata secondo le dimensioni w e h. per
	 * lasciare inalterata una dimensione, utilizzare -1. Il file dell'icona
	 * deve essere in gui.img
	 * 
	 * @param name
	 *            il nome del file dell'icona
	 * @param w
	 *            la larghezza dell'immagine da ritornare, oppure -1 per
	 *            lasciare quella originale
	 * @param h
	 *            l'altezza dell'immagine da ritornare, oppure -1 per lasciare
	 *            quella originale
	 * @return una ImageIcon secondo i parametri specificati
	 */
	public static ImageIcon getIcon(String name, int w, int h) {
		URL url = ClassLoader.getSystemClassLoader().getResource("gui/img/"+name);
		Image i = Toolkit.getDefaultToolkit().getImage(url);
		i = i.getScaledInstance(w, h, Image.SCALE_SMOOTH);
		return new ImageIcon(i);
	}
	
	/**
	 * Mostra un messaggio di errore. Utile per mostrare un'eccezione.
	 * @param messaggio il messaggio da mostrare.
	 * @param e l'eccezione lanciata.
	 */
	@SuppressWarnings("serial")
	public static void dialogoException(String messaggio, final Throwable e) {
		JPanel pan = new JPanel();
		
		pan.add(new JLabel(messaggio + " Per info sull'errore clicca"));
		
		LinkLabel link = new LinkLabel("qui");
		pan.add(link);
		
		link.setAction(new AbstractAction() {
			public void actionPerformed(ActionEvent event) {
				
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				PrintStream ps = new PrintStream(baos);
				e.printStackTrace(ps);
				
				JTextArea ta = new JTextArea();
				ta.setEditable(false);
				ta.setMargin(new Insets(5, 5, 5, 5));
				ta.append(e.getMessage() + "\n");
				ta.append(baos.toString());
				ta.setCaretPosition(ta.getText().length());
				
				JScrollPane sp = new JScrollPane(ta);
				sp.setPreferredSize(new Dimension(400, 300));
				
				JDialog d = new JDialog();
				d.setTitle("Informazioni sull'eccezione");
				d.setModal(true);
				d.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
				d.add(sp);
				d.pack();
				d.setLocation(UTIL.centerOnScreen(d.getSize()));
				d.setVisible(true);
			}
		});
		errMsg(null, pan);
	}
	
	/**
	 * mostra un {@link JOptionPane} con un messaggio di errore
	 * @param p
	 * @param m
	 */
	public static void errMsg(Component p, Object m) {
		JOptionPane.showMessageDialog(p, 
				m, 
				"Errore", 
				JOptionPane.ERROR_MESSAGE);
	}
	
	public static void showSettingsDialog(){
		final JDialog d = new JDialog();
		d.setTitle("Impostazioni - BodeDiagramPlotter");
		d.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		d.setModal(true);
		
		JLabel label1 = new JLabel("Colore grafico reale:");
		final ColorButton colorReale = new ColorButton(COLORE_FUNZIONE_REALE);
		JLabel label2 = new JLabel("Colore grafico asintotico:");
		final ColorButton colorAsint = new ColorButton(COLORE_FUNZIONE_ASINT);
		final JCheckBox disegnaGReale = new JCheckBox("Disegna grafico reale", DISEGNA_GRAFICO_REALE);
		final JCheckBox disegnaGAsint = new JCheckBox("Disegna grafico asintotico", DISEGNA_GRAFICO_ASINTOTICO);
		JLabel labelOrdine = new JLabel("Livelli di tracciamento:");
		final JRadioButton asintSotto = new JRadioButton("Grafico asintotico sotto, grafico reale sopra", LIVELLO_ASINT_SOTTO);
		JRadioButton realeSotto = new JRadioButton("Grafico reale sotto, grafico asintotico sopra", !LIVELLO_ASINT_SOTTO);
		ButtonGroup gb = new ButtonGroup();
		gb.add(asintSotto);
		gb.add(realeSotto);
		JButton ok = new JButton("Conferma modifiche");
		
		label1.setSize(label1.getPreferredSize());
		colorReale.setSize(colorReale.getPreferredSize());
		label2.setSize(label2.getPreferredSize());
		colorAsint.setSize(colorAsint.getPreferredSize());
		disegnaGReale.setSize(disegnaGReale.getPreferredSize());
		disegnaGAsint.setSize(disegnaGAsint.getPreferredSize());
		labelOrdine.setSize(labelOrdine.getPreferredSize());
		asintSotto.setSize(asintSotto.getPreferredSize());
		realeSotto.setSize(realeSotto.getPreferredSize());
		ok.setSize(ok.getPreferredSize());
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				if(!disegnaGAsint.isSelected() && !disegnaGReale.isSelected()){
					errMsg(null, "Selezionare almeno un grafico da tracciare, reale o asintotico.");
					return;
				}
				COLORE_FUNZIONE_ASINT = colorAsint.c;
				COLORE_FUNZIONE_REALE = colorReale.c;
				DISEGNA_GRAFICO_ASINTOTICO = disegnaGAsint.isSelected();
				DISEGNA_GRAFICO_REALE = disegnaGReale.isSelected();
				LIVELLO_ASINT_SOTTO = asintSotto.isSelected();
				d.dispose();
			}
		});
		colorAsint.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Color c = JColorChooser.showDialog(null, "Colore per il grafico asintotico", colorAsint.c);
				if(c!=null){
					colorAsint.c=c;
				}
			}
		});
		
		colorReale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				Color c = JColorChooser.showDialog(null, "Colore per il grafico reale", colorReale.c);
				if(c!=null){
					colorReale.c=c;
				}
			}
		});
		
		int pWidth = 400;
		int margineOrizzontale = 20;
		int margineVerticale = 20;
		int interlinea = 5;
		int xButt = 180;
		int x = margineOrizzontale;
		int y = margineVerticale;
		
		label1.setLocation(x, y);
		x=xButt;
		colorReale.setLocation(x, y-4);
		
		y+=colorReale.getHeight()+interlinea;
		x=margineOrizzontale;
		label2.setLocation(x, y);
		x=xButt;
		colorAsint.setLocation(x, y-4);
		
		y+=colorAsint.getHeight()+interlinea*3;
		x=margineOrizzontale;
		disegnaGReale.setLocation(x, y);
		
		y+=disegnaGReale.getHeight();
		disegnaGAsint.setLocation(x, y-4);
		
		y+=disegnaGAsint.getHeight()+interlinea*3;
		labelOrdine.setLocation(x, y);
		
		y+=labelOrdine.getHeight();
		asintSotto.setLocation(x, y);
		
		y+=asintSotto.getHeight();
		realeSotto.setLocation(x, y-4);
		
		y+= 8*interlinea;
		int okLocationX = pWidth-margineOrizzontale-ok.getWidth();
		ok.setLocation(okLocationX, y);
		
		y+= 4*interlinea+margineVerticale;
		
		JPanel p = new JPanel(null);
		p.setPreferredSize(new Dimension(pWidth, y));
		p.add(label1);
		p.add(colorReale);
		p.add(label2);
		p.add(colorAsint);
		p.add(disegnaGReale);
		p.add(disegnaGAsint);
		p.add(labelOrdine);
		p.add(asintSotto);
		p.add(realeSotto);
		p.add(ok);
		
		d.setContentPane(p);
		d.pack();
		d.setLocationRelativeTo(null);
		d.setVisible(true);
		
	}
	
	
}

@SuppressWarnings("serial")
class ColorButton extends JButton{
	public Color c;
	public ColorButton(final Color defColor){
		c = defColor;
		JLabel l = new JLabel("ciao"){
			protected void paintComponent(Graphics g) {
				g.setColor(c);
				g.fillRect(0, 0, getWidth(), getHeight());
			}
		};
		l.setPreferredSize(new Dimension(15,15));
		add(l);
	}
	
}













