package fdt;

import static app.UTIL.DB;
import static java.lang.Math.pow;

import java.util.LinkedList;

public class FDT {

	/**
	 * guadagno statico
	 */
	private double k;

	/**
	 * numero di zeri/poli nell'origine.
	 */
	private int h;

	/**
	 * I fattori di questa FDT
	 */
	private LinkedList<Fattore> fattori;

	public FDT(double k, int h) {
		this.k = k;
		this.h = h;
		fattori = new LinkedList<Fattore>();
	}

	public void aggiungiFattore(Fattore f) {
		fattori.add(f);
	}

	/**
	 * valuta il modulo della funzione alla pulsazione w
	 * 
	 * @param w
	 *            la pulsazione alla quale valutare il modulo
	 * @return il modulo in decibel della funzione valutata per w
	 */
	public double valutaModuloDB(double w) {

		double moduloFattori = 0;
		for(Fattore i : fattori){
			moduloFattori += i.valutaModuloDB(w);
		}
		return DB(Math.abs(k)) + DB(pow(w, h)) + moduloFattori;
	}

	/**
	 * valuta la fase della funzione alla pulsazione w
	 * 
	 * @param w
	 *            la pulsazione alla quale valutare la fase
	 * @return la fase in gradi della funzione valutata per w
	 */
	public double valutaFase(double w) {

		double faseK = k < 0 ? -180 : 0;
		double faseZPOrig = 90 * h;

		double faseFattori = 0;
		for(Fattore i : fattori){
			faseFattori += i.valutaFase(w);
		}
		return faseK + faseZPOrig + faseFattori;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for(Fattore i : fattori){
			sb.append(i.toString()+"\n");
		}
		return
			"k="+k+"\n" +
			"h="+h+"\n" +
			sb.toString();
	}
	
	public double getGuadagnoStatico(){
		return k;
	}
	
	public int getZeriPoliOrigine(){
		return h;
	}
	
	public LinkedList<Fattore> getFattori(){
		return fattori;
	}

}
