package fdt;

import static app.UTIL.DB;
import static app.UTIL.fase;
import static java.lang.Math.pow;
import static java.lang.Math.sqrt;

/**
 *  un fattore nella forma (1 + a*s + b*s^2)^m. Se m<0 questo fattore
 * rappresenta una coppia di poli C+C. Se m>0 invece questo fattore rappresenta
 * una coppia di zeri C+C.
 * 
 * @author Devil_Dj
 * 
 */
public class FattCC extends Fattore {

	private Frazione a;
	private Frazione b;
	private int m;

	public FattCC(Frazione a, Frazione b, int m) {
		this.a = a;
		this.b = b;
		this.m = m;
	}

	
	public double valutaModuloDB(double w) {
		// modulo = rad( (1-bw^2)^2 + (aw)^2 )
		double Re = 1 - b.val() * pow(w, 2);
		double Im = a.val() * w;

		double modulo = sqrt(pow(Re, 2) + pow(Im, 2));
		return m * DB(modulo);
	}
	
	
	public double valutaFase(double w){
		if(m==0) return 0;
		double Re = 1 - b.val() * pow(w, 2);
		double Im = a.val() * w;
		return m*fase(Re, Im);
	}
	
	@Override
	public String toString() {
		return "fcc, a="+
			a.num + "/" + a.den + "," +
			"b=" + 
			b.num + "/" + b.den + "," +
			"m="+m;
	}

}























