package fdtBuilder;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.StringTokenizer;

import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import app.UTIL;
import fdt.FDT;
import fdt.FattCC;
import fdt.FattSemplice;
import fdt.Fattore;
import fdt.Frazione;
import gui.LinkLabel;

public class Builder {
	
	private String spec;
	
	private double k;
	private int h;
	private LinkedList<Fattore> fattori;
	
	private StringBuilder sb;
	private int lineNumber;
	private String elementoCorrente;
	
	public Builder(String spec){
		init(spec);
	}
	
	public Builder(File f) throws FileNotFoundException{
		Scanner sc = new Scanner(f);
		StringBuilder sb = new StringBuilder();
		while(sc.hasNextLine()){
			sb.append(sc.nextLine()+"\n");
		}
		init(sb.toString());
	}
	
	private void init(String spec){
		this.spec=spec;
		
		k = 1;
		h = 0;
		fattori = new LinkedList<Fattore>();
		
		sb = new StringBuilder();
		lineNumber = 0;
		
	}
	
	
	public FDT build(){
		StringTokenizer st = new StringTokenizer(spec, "\n", false);
		while(st.hasMoreTokens()){
			lineNumber++;
			elementoCorrente = st.nextToken();
			elemento();
		}
		FDT fdt = new FDT(k, h);
		for(Fattore i : fattori){
			fdt.aggiungiFattore(i);
		}
		mostraErrori();
		return fdt;
	}
	
	private void elemento(){
		String elem = elementoCorrente.replaceAll(" ", "").toLowerCase();
		if(elem.startsWith("k=")){
			guadagno(elem.substring(2));
		}else if(elem.startsWith("h=")){
			tipo(elem.substring(2));
		}else if(elem.startsWith("fs")){
			fs(elem.substring(2));
		}else if(elem.startsWith("fcc")){
			fcc(elem.substring(3));
		}else{
			errore("Elemento non formattato correttamente");
		}
	}
	
	private void guadagno(String s){
		try{
			double k = Double.parseDouble(s);
			this.k=k;
		}catch(Exception e){
			errore("Guadagno (k) non formattato correttamente");
		}
	}
	
	private void tipo(String s){
		try{
			int h = Integer.parseInt(s);
			this.h=h;
		}catch(Exception e){
			errore("Tipo del sistema (h) non formattato correttamente");
		}
	}
	
	private void fs(String s){
		if(s.startsWith(",")){
			s=s.substring(1);
		}else{
			errore("Fattore semplice non formattato correttamente");
			return;
		}
		try{
			StringTokenizer st = new StringTokenizer(s, ",", false);
			String a = st.nextToken();
			String m = st.nextToken();
			Frazione f1 = fract(a);
			if(a==null){
				errore("Parametro a del fattore semplice non formattato correttamente");
				return;
			}
			try{
				int esp = Integer.parseInt(m.substring(2));
				fattori.add(new FattSemplice(f1, esp));
			}catch(Exception e){
				errore("Parametro m del fattore semplice non formattato correttamente");
			}
		}catch(NoSuchElementException e){
			errore("Fattore semplice non formattato correttamente");
		}
	}
	
	private void fcc(String s){
		if(s.startsWith(",")){
			s=s.substring(1);
		}else{
			errore("Fattore complesso non formattato correttamente");
			return;
		}
		try{
			StringTokenizer st = new StringTokenizer(s, ",", false);
			String a = st.nextToken();
			String b = st.nextToken();
			String m = st.nextToken();
			Frazione f1 = fract(a);
			if(f1==null){
				errore("Parametro a del fattore complesso non formattato correttamente");
				return;
			}
			Frazione f2 = fract(b);
			if(f2==null){
				errore("Parametro b del fattore complesso non formattato correttamente");
				return;
			}
			try{
				int esp = Integer.parseInt(m.substring(2));
				fattori.add(new FattCC(f1, f2, esp));
			}catch(Exception e){
				e.printStackTrace();
				errore("Parametro m del fattore complesso non formattato correttamente");
			}
		}catch(NoSuchElementException e){
			errore("Fattore fattore complesso non formattato correttamente");
		}
		
	}
	
	private Frazione fract(String s){
		try{
			s=s.substring(2);
			int x = s.indexOf("/");
			if(x>0){
				String sNum = s.substring(0, x);
				String sDen = s.substring(x+1);
				double num = Double.parseDouble(sNum);
				double den = Double.parseDouble(sDen);
				return new Frazione(num, den);
			}else{
				double num = Double.parseDouble(s);
				return new Frazione(num, 1);
			}
		}catch(Exception e){
			errore("Frazione non formattata correttamente");
			return null;
		}
	}
	
	private void errore(String descr) {
		String log = "line "+lineNumber+ ": " + 
					descr + " ("+elementoCorrente+")";
		sb.append(log+"\n");
	}
	
	@SuppressWarnings("serial")
	private void mostraErrori(){
		
		final String errors = sb.toString();
		if(errors.length()==0) return;
		
		JPanel pan = new JPanel();

		pan.add(new JLabel("Sono stati rilevati errori di compilazione. Per maggiori dettagli clicca"));

		LinkLabel link = new LinkLabel("qui");
		pan.add(link);

		link.setAction(new AbstractAction() {
			public void actionPerformed(ActionEvent event) {
				JTextArea ta = new JTextArea();
				ta.setEditable(false);
				ta.setText(errors+"\n");
				ta.setCaretPosition(ta.getText().length());
				ta.setMargin(new Insets(5, 5, 5, 5));

				JScrollPane sp = new JScrollPane(ta);
				sp.setPreferredSize(new Dimension(400, 300));
				
				JDialog d = new JDialog();
				d.setTitle("Errori di compilazione.");
				d.setModal(true);
				d.add(sp);
				d.pack();
				d.setLocation(UTIL.centerOnScreen(d.getSize()));
				d.setVisible(true);
			}
		});
		UTIL.errMsg(null, pan);
	}

}
