package gui;

import gui.pannelliDiag.PannelloDiagramma;

import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import app.UTIL;

@SuppressWarnings("serial")
public final class ExportHelper {

	private static JFileChooser fc;

	private ExportHelper() { // non  necessario istanziare questa classe
	}

	static {
		fc = new JFileChooser() {
			@Override
			public void approveSelection() {
				if (getSelectedFile().exists()) {
					int x = JOptionPane.showConfirmDialog(this,
							"Il file selezionato  esistente. Sovrascrivere?",
							"Conferma", JOptionPane.YES_NO_OPTION);
					if (!(x == JOptionPane.YES_OPTION)) {
						return;
					}
				}
				super.approveSelection();
			}
		};

		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.setAcceptAllFileFilterUsed(false);
		FileNameExtensionFilter fJPG = new FileNameExtensionFilter(
				"Immagine di tipo JPEG (*.jpg)", "jpg");
		FileNameExtensionFilter fPNG = new FileNameExtensionFilter(
				"Immagine di tipo PNG (*.png)", "png");
		FileNameExtensionFilter fGIF = new FileNameExtensionFilter(
				"Immagine di tipo GIF (*.gif)", "gif");
		FileNameExtensionFilter fBMP = new FileNameExtensionFilter(
				"Immagine di tipo Bitmap (*.bmp)", "bmp");

		fc.addChoosableFileFilter(fPNG);
		fc.addChoosableFileFilter(fGIF);
		fc.addChoosableFileFilter(fJPG);
		fc.addChoosableFileFilter(fBMP);
		fc.setFileFilter(fPNG);
	}

	public static void mostraDialogo(final PannelloDiagramma p) {

		try {

			if (!(fc.showSaveDialog(p) == JFileChooser.APPROVE_OPTION)) {
				return;
			}

			FileNameExtensionFilter ff = (FileNameExtensionFilter) fc
					.getFileFilter();
			String ext = ff.getExtensions()[0];

			File selectedFile = fc.getSelectedFile();

			String completeName = selectedFile.getAbsolutePath();
			if (!completeName.endsWith(ext)) {
				selectedFile = new File(completeName.concat("." + ext));
			}

			if (selectedFile.exists()) {
				if (!selectedFile.delete()) {
					UTIL.errMsg(p, "Impossibile sovrascrivere il file.");
					return;
				}
			}

			// verifico che esista la struttura di directory contenente il file
			if (!selectedFile.getParentFile().exists()) {
				if (!selectedFile.getParentFile().mkdirs()) {
					UTIL.errMsg(p, "Errore nella creazione della struttura "
							+ "di directory contenente il file.");
					return;
				}
			}

			if (!selectedFile.createNewFile()) {
				UTIL.errMsg(p, "Impossibile creare il file.");
				return;
			}

			// a questo punto il file  stato creato ed  disponibile per la
			// scrittura

			if (!creaScreenshot(p, selectedFile, ext)) {
				throw new RuntimeException(
						"Valore di ritorno creaScreenshot() false");
			}
			
			JOptionPane.showMessageDialog(p, "Immagine esportata.");

		} catch (final Exception exc) {
			UTIL.dialogoException("Errore durante la creazione "
					+ "dell'immagine. Il file potrebbe non essere stato "
					+ "creato correttamente.", exc);
		}
	}

	/**
	 * Crea uno screenshot del componente ricevuto come parametro e lo salva nel
	 * file "dest". Il formato di salvataggio va indicato nella stringa "format"
	 * e pu essere jpg, png, gif, bmp.
	 * 
	 * @param component
	 *            il componente di cui creare lo screenshot
	 * @param dest
	 *            il file su cui scrivere l'immagine
	 * @param format
	 *            il formato dell'immagine
	 * @return true se l'operazione  andata a buon fine
	 * @throws IOException
	 *             se si verifica qualche errore nella creazione/scrittura.
	 */
	public static boolean creaScreenshot(Component component, File dest,
			String format) throws IOException {
		BufferedImage image = (BufferedImage) component.createImage(component
				.getWidth(), component.getHeight());
		Graphics2D tGraphic = image.createGraphics();
		component.paint(tGraphic);
		return ImageIO.write(image, format, dest);
	}

}
