package gui;

import fdt.FDT;
import fdtBuilder.Builder;
import gui.pannelliDiag.PannelloDiagramma;
import gui.pannelliGestione.PannelloControllo;
import gui.pannelliGestione.PannelloFunzione;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;

import app.UTIL;

@SuppressWarnings("serial")
public class MainWindow extends JFrame {
	
	private DiagramWindow diagW;
	
	private JPanel root;
	private PannelloFunzione pFunzione;
	private PannelloControllo pControllo;
	
	private JButton traccia, esci, impostazioni;
	
	public MainWindow() {
		super("BodeDiagram plotter v2.1 beta - by De Meco Andrea");
		
		diagW = new DiagramWindow(this);
		root = new JPanel(null);
		pFunzione = new PannelloFunzione();
		pControllo = new PannelloControllo(diagW);
		
		traccia = new JButton("Traccia grafico");
		traccia.setSize(traccia.getPreferredSize());
		traccia.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				traccia();
			}
		});
		
		esci = new JButton("Esci");
		esci.setSize(esci.getPreferredSize());
		esci.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		
		impostazioni = new JButton("Impostazioni");
		impostazioni.setSize(impostazioni.getPreferredSize());
		impostazioni.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				UTIL.showSettingsDialog();
				diagW.repaint();
			}
		});
		
		posizionaElementi();
		
		root.add(pFunzione);
		root.add(pControllo);
		root.add(traccia);
		root.add(esci);
		root.add(impostazioni);
		
		setContentPane(root);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setResizable(false);
		pack();
		setLocation(UTIL.centerOnScreen(getSize()));
		
	}
	
	private void posizionaElementi(){
		int wMargin = 10;
		int wTot = 480;
		
		int interlinea = 15;
		
		int hMargin = 10;
		int hPFunzione = 430;
		int hPControllo = 130;
		int hButtons = esci.getHeight();
		
		int yPFunzione = hMargin;
		int yPControllo = yPFunzione + hPFunzione + interlinea;
		int yButtons = yPControllo + hPControllo + interlinea;
		
		int wRoot = wMargin + wTot + wMargin;
		int hRoot = hMargin + hPFunzione + 
			interlinea + hPControllo + 
			interlinea + hButtons + hMargin;
		
		root.setSize(wRoot, hRoot);
		root.setPreferredSize(root.getSize());
		root.setBorder(BorderFactory.createEtchedBorder());
		
		pFunzione.setBounds(wMargin, yPFunzione, wTot, hPFunzione);
		pControllo.setBounds(wMargin, yPControllo, wTot, hPControllo);
		
		int xEsci = wRoot - wMargin - esci.getWidth();
		int xTraccia = xEsci - interlinea - traccia.getWidth();
		
		esci.setLocation(xEsci, yButtons);
		traccia.setLocation(xTraccia, yButtons);
		impostazioni.setLocation(wMargin, yButtons);
	}
	
	private void traccia(){
		String fdtDescr = pFunzione.getFunzione();
		FDT fdt = new Builder(fdtDescr).build();
		PannelloDiagramma diag = new PannelloDiagramma(fdt);
		diagW.aggiungiDiagramma(diag);
		pControllo.updateObserverScalaGrafico();
		if(!diagW.isVisible()){
			posizionaFinestre();
			diagW.setVisible(true);
		}
	}
	
	private void posizionaFinestre(){
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int wDiagW = screen.width - getWidth() - 100;
		Point p = UTIL.centerOnScreen(new Dimension(getWidth()+wDiagW, getHeight()));
		setLocation(p);
		diagW.setBounds(p.x+getWidth(), p.y, wDiagW, getHeight());
	}
	
	public void diagWClosed(){
		setLocation(UTIL.centerOnScreen(getSize()));
	}
	
	public static void main(String[] args) {
		try{
//			for(LookAndFeelInfo i : UIManager.getInstalledLookAndFeels()){
//				UIManager.setLookAndFeel(i.getClassName());
//				new MainWindow().setVisible(true);
//			}
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			new MainWindow().setVisible(true);
		}catch (Throwable e) {
			UTIL.dialogoException("Si  verificato un errore imprevisto.", e);
		}
	}

}









