package gui.pannelliDiag;
import static app.UTIL.H_LABEL_ORIZ;
import static app.UTIL.LABEL_FONT;
import static app.UTIL.LEFT_BORDER;
import static app.UTIL.LOWER_BORDER;
import static app.UTIL.MIDDLE_BORDER;
import static app.UTIL.PANEL_BACKGR;
import static app.UTIL.RIGHT_BORDER;
import static app.UTIL.UPPER_BORDER;
import static app.UTIL.W_LABEL_VERT;
import fdt.FDT;

import java.awt.Dimension;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JLabel;
import javax.swing.JPanel;

import app.UTIL;


@SuppressWarnings("serial")
public class PannelloDiagramma extends JPanel {
	
	private PannelloDiagrammaModuliAsint pm;
	private PannelloDiagrammaFasi pf;
	private JPanel labelModuli;
	private JPanel labelFase;
	private JPanel labelPuls;
	
	private JLabel[] labels;
	
	public PannelloDiagramma(FDT funzione){
		super(null);
		
		labels = new JLabel[6];
		labels[0] = new JLabel("Bode diagram");
		labels[0].setHorizontalAlignment(JLabel.CENTER);
		labels[1] = new JLabel("Margine di fase: [n/d]"); //label del margine di fase
		labels[2] = new JLabel("Margine di ampiezza: [n/d]"); //label del margine di amplitude
		labels[3] = new JLabel(new VTextIcon(this, "Modulo (dB)", 2));
		labels[4] = new JLabel(new VTextIcon(this, "Fase (gradi)", 2));
		labels[5] = new JLabel("Frequenza (log(w))");
		labels[5].setHorizontalAlignment(JLabel.CENTER);
		
		labelModuli = new JPanel(null);
		labelFase = new JPanel(null);
		labelPuls = new JPanel(null);
		labelModuli.setBackground(PANEL_BACKGR);
		labelFase.setBackground(PANEL_BACKGR);
		labelPuls.setBackground(PANEL_BACKGR);
		
		pm = new PannelloDiagrammaModuliAsint(funzione, this);
		pf = new PannelloDiagrammaFasi(funzione, this);
		
		add(pm);
		add(pf);
		add(labelModuli);
		add(labelFase);
		add(labelPuls);
		
		for(int i=0; i<labels.length; i++){
			labels[i].setFont(LABEL_FONT);
			add(labels[i]);
		}
		
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				regolaPosizioni();
			}
		});
		
	}
	
	public void aggiungiNumerettoAsseModuli(String val, int y) {
		UTIL.aggiungiNumerettoV(labelModuli, val, 0, y);
		labelModuli.repaint();
	}
	public void aggiungiNumerettoAsseFase(String val, int y) {
		UTIL.aggiungiNumerettoV(labelFase, val, 0, y);
		labelFase.repaint();
	}
	public void aggiungiNumerettoPulsazione(String val, int x) {
		UTIL.aggiungiNumerettoO(labelPuls, val, x, 0);
		labelPuls.repaint();
	}
	
	public void azzeraLabelAsseModulo(){
		labelModuli.removeAll();
	}
	public void azzeraLabelAsseFase(){
		labelFase.removeAll();
	}
	public void azzeraLabelPulsazione(){
		labelPuls.removeAll();
	}
	
	
	public void setLabelMargineFase(String value){
		labels[1].setText(value);
	}
	public void setLabelMargineAmpiezza(String value){
		labels[2].setText(value);
	}
	
	public void aggiornaScalaGrafico(int numDec, int logwMin, double fMax, double fMin, double mMax, double mMin){
		pm.setLogwMin(logwMin);
		pm.setLogwMax(logwMin+numDec);
		pm.setmMin(mMin);
		pm.setmMax(mMax);
		
		pf.setLogwMin(logwMin);
		pf.setLogwMax(logwMin+numDec);
		pf.setfMin(fMin);
		pf.setfMax(fMax);
		
		repaint();
	}
	
	
	private void regolaPosizioni(){
		
		Dimension d = getSize();
		int labH = 18;
		int yLab1 = 10;
		int yLab2 = 35;
		int yLab3 = 55;
		
		labels[0].setBounds(0, yLab1, d.width, labH);
		labels[1].setBounds(LEFT_BORDER+W_LABEL_VERT+50, yLab2, d.width-LEFT_BORDER-RIGHT_BORDER, labH);
		labels[2].setBounds(LEFT_BORDER+W_LABEL_VERT+50, yLab3, d.width-LEFT_BORDER-RIGHT_BORDER, labH);
		
		//altezza di ciascuno dei grafici
		int hGraf = (d.height - UPPER_BORDER - MIDDLE_BORDER - LOWER_BORDER - H_LABEL_ORIZ)/2;
		//larghezza di ciascuno dei 2 grafici
		int wGraf = d.width - RIGHT_BORDER - LEFT_BORDER - W_LABEL_VERT;
		
		labels[3].setBounds(10, UPPER_BORDER, labH, hGraf);
		
		//dove inizia il secondo grafico
		int y2 = UPPER_BORDER + hGraf + MIDDLE_BORDER;
		
		labels[4].setBounds(10, y2, labH, hGraf);
		
		//dove iniziano i 2 grafici
		int xGraf = LEFT_BORDER + W_LABEL_VERT;
		
		labels[5].setBounds(xGraf, d.height-10-labH, wGraf, labH);
		
		labelModuli.setBounds(LEFT_BORDER, UPPER_BORDER, W_LABEL_VERT, hGraf);
		labelFase.setBounds(LEFT_BORDER, y2, W_LABEL_VERT, hGraf+H_LABEL_ORIZ);
		labelPuls.setBounds(xGraf, y2+hGraf, wGraf, H_LABEL_ORIZ);
		
		pm.setBounds(xGraf, UPPER_BORDER, wGraf, hGraf);
		pf.setBounds(xGraf, y2, wGraf, hGraf);
		
	}

}





























