package gui.pannelliDiag;
import static app.UTIL.ASSE_PRINCIPALE;
import static app.UTIL.ASSE_SECONDARIO;
import static app.UTIL.COLORE_FUNZIONE_ASINT;
import static app.UTIL.COLORE_FUNZIONE_REALE;
import static app.UTIL.F_MAX_DEFAULT;
import static app.UTIL.F_MIN_DEFAULT;
import static app.UTIL.LOGWMAX_DEFAULT;
import static app.UTIL.LOGWMIN_DEFAULT;
import static app.UTIL.MASSIMA_F_MIN;
import static app.UTIL.MINIMA_F_MAX;
import static app.UTIL.PANEL_BACKGR;
import static app.UTIL.formatDouble;
import static app.UTIL.trovaEsponenteNS;
import static java.lang.Math.log10;
import static java.lang.Math.pow;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JPanel;

import app.UTIL;
import fdt.FDT;
import fdt.FattSemplice;
import fdt.Fattore;


@SuppressWarnings("serial")
public class PannelloDiagrammaFasi extends JPanel {
	
	private int logwMin, logwMax;
	private double fMin, fMax;
	
	private FDT funzione;
	private PannelloDiagramma finestra;
	
	public PannelloDiagrammaFasi(FDT funzione, PannelloDiagramma parent) {
		super(null);
		this.funzione=funzione;
		this.finestra = parent;
		setfMin(F_MIN_DEFAULT);
		setfMax(F_MAX_DEFAULT);
		setLogwMin(LOGWMIN_DEFAULT);
		setLogwMax(LOGWMAX_DEFAULT);
		setBackground(PANEL_BACKGR);
	}
	
	//------------ METODI MUTATORI-ACCESSORI --------------------
	public void setLogwMin(int logwMin) {
		this.logwMin = logwMin;
	}
	public void setLogwMax(int logwMax) {
		this.logwMax = logwMax;
	}
	public void setfMin(double fMin) {
		if(fMin>MASSIMA_F_MIN) throw new IllegalArgumentException("Deve essere fMin<"+MASSIMA_F_MIN);
		this.fMin = fMin;
	}
	public void setfMax(double fMax) {
		if(fMax<MINIMA_F_MAX) throw new IllegalArgumentException("Deve essere fMax>"+MINIMA_F_MAX);
		this.fMax = fMax;
	}
	//-----------------------------------------------------------
	
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		finestra.azzeraLabelAsseFase();
		disegnaAsseOrdinate(g);
		finestra.azzeraLabelPulsazione();
		disegnaAsseAscisse(g);
		disegnaAsse180(g);
		disegnaCornice(g);
		disegnaFunzione(g);
	}
	
	private void disegnaCornice(Graphics g) {
		g.setColor(Color.black);
		g.drawRect(0, 0, getWidth()-1, getHeight()-1);
	}
	
	private void disegnaAsseAscisse(Graphics g) {
		
		// questa funzione deve anche segnare le label della punsazione
		
		// disegno le linee in corrispondenza delle decadi
		for (int i = logwMin; i <= logwMax; i++) {
			double valX = pow(10, i);
			int cX = trovaX(valX);
			if (cX < 0)
				continue;
			g.setColor(ASSE_PRINCIPALE);
			g.drawLine(cX, 0, cX, getHeight()-1);
			if(i<logwMax) finestra.aggiungiNumerettoPulsazione(i+"", cX);

			// disegno le lineee in corrisp dei numeretti meno importanti
			for (int j = 2; j <= 9; j++) {
				double valx1 = j * valX;
				cX = trovaX(valx1);
				if (cX < 0)
					continue;
				g.setColor(ASSE_SECONDARIO);
				g.drawLine(cX, 1, cX, getHeight()-2);
			}
		}
		
		// disegno la linea dell'asse
		int cY0 = trovaY(0);
		g.setColor(ASSE_PRINCIPALE);
		g.drawLine(0, cY0, getWidth() - 1, cY0);
		
	}
	
	
	private void disegnaAsseOrdinate(Graphics g){
		//distanza in pixel tra le label
		int dp = UTIL.DISTANZA_NUMERETTI_ASSE_ORDINATE;
		//numero di intervallini
		int n = getHeight()/dp;
		//range dei valori disegnabili
		int range = (int) (fMax - fMin);
		//distanza in valore tra una label e l'altra
		int intervallo = range/n;
		//intervallo arrotondato a 5
		int interv = 5 * (intervallo / 5);
		
		g.setColor(ASSE_SECONDARIO);
		
		//disegno quelli sopra l'asse
		int val = 0;
		
		while(true){
			val += interv;
			int y = trovaY(val);
			if(y==-1) break;
			//disegno l'asse
			g.drawLine(1, y, getWidth()-2, y);
			//aggiungo il numeretto
			finestra.aggiungiNumerettoAsseFase(val+"", y);
		}
		
		//disegno quelli sotto l'asse
		val = 0;
		while(true){
			val -= interv;
			int y = trovaY(val);
			if(y==-1) break;
			//disegno l'asse
			g.drawLine(1, y, getWidth()-2, y);
			//disegno il numeretto
			finestra.aggiungiNumerettoAsseFase(val+"", y);
		}
		
		//aggiungo lo zero
		int y0 = trovaY(0);
		finestra.aggiungiNumerettoAsseFase(0+"", y0);
		
	}
	
	private void disegnaAsse180(Graphics g){
		int y180 = trovaY(-180);
		finestra.aggiungiNumerettoAsseFase(-180+"", y180);
		g.setColor(Color.red);
		g.drawLine(1, y180, getWidth()-2, y180);
	}
	
	private void disegnaFunzione(Graphics g){
		
		//il valore di w per ogni pixel
		double[] wVals = new double[getWidth()];
		for(int i=0; i<wVals.length; i++){
			wVals[i]=revTrovaX(i);
		}
		
		//il valore della funzione asint in db per ogni w dei pixel
		double[] valsAsint = calcolaArrayValoriAsint(wVals);
		
		//il valore della funzione reale in db per ogni w dei pixel
		double[] valsReale = calcolaArrayValoriReali(wVals);
		
		
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		
		if(UTIL.LIVELLO_ASINT_SOTTO){
			//disegno prima il grafico asintotico
			g.setColor(COLORE_FUNZIONE_ASINT);
			if(UTIL.DISEGNA_GRAFICO_ASINTOTICO){
				disegnaGrafico(g, valsAsint);
			}
			g.setColor(COLORE_FUNZIONE_REALE);
			if(UTIL.DISEGNA_GRAFICO_REALE){
				disegnaGrafico(g, valsReale);
			}
		}else{
			g.setColor(COLORE_FUNZIONE_REALE);
			if(UTIL.DISEGNA_GRAFICO_REALE){
				disegnaGrafico(g, valsReale);
			}
			g.setColor(COLORE_FUNZIONE_ASINT);
			if(UTIL.DISEGNA_GRAFICO_ASINTOTICO){
				disegnaGrafico(g, valsAsint);
			}
		}
		
		//PARTE VECCHIA
		
		g.setColor(COLORE_FUNZIONE_REALE);
		
		double w = revTrovaX(0);
		double val1 = funzione.valutaFase(w);
		
		int xStart = -1;
		int xEnd = -1;
		int y180 = trovaY(-180);
		
		for(int i=1; i<getWidth(); i++){
			w = revTrovaX(i);
			double val2 = funzione.valutaFase(w);
			if(val2>fMin && val2<fMax){
				//g.drawLine(i-1, trovaY(val1), i, trovaY(val2));
				
				//per il calcolo della pulsazione di marg.amp
				if(trovaY(val1)==y180){
					if(xStart==-1) xStart = i-1;
					xEnd = i-1;
				}
			}
			val1=val2;
		}
		
		calcolaWma(xStart, xEnd);
	}

	private void disegnaGrafico(Graphics g, double[] vals){
		int x1 = 0;
		int y1 = trovaY(vals[0]);
		for(int i=1; i<getWidth(); i++){
			int y2 = trovaY(vals[i]);
			if(y2!=y1){
				g.drawLine(x1, y1, i, y2);
				x1=i;
				y1=y2;
			}
		}
		g.drawLine(x1, y1, getWidth()-1, trovaY(vals[getWidth()-1]));
	}

	private double[] calcolaArrayValoriAsint(double[] wVals) {
		//calcolo dell'incremento (45 gradi per decade)
		double numeroDecadi = logwMax - logwMin;
		double pixPerDecade = (double) getWidth() / numeroDecadi;
		double incremento = 45/pixPerDecade;
		
		double[] vals = new double[wVals.length];
		for(int i=0; i<wVals.length; i++){
			vals[i]+=90*funzione.getZeriPoliOrigine();
			
			for(Fattore fattore : funzione.getFattori()){
				if (fattore instanceof FattSemplice) {
					FattSemplice fs = (FattSemplice) fattore;
					int wUnaDecadePrima = trovaX(1/(fs.getA()*10));
					int wUnaDecadeDopo = trovaX(10/fs.getA());
					
					if(i>wUnaDecadePrima){
						if(i<wUnaDecadeDopo){
							vals[i] += incremento*(i-wUnaDecadePrima)*fs.getM()*Math.signum(fs.getA());
						}else{
							vals[i] += 90*fs.getM()*Math.signum(fs.getA());
						}
					}
				}
			}
		}
		return vals;
	}

	private double[] calcolaArrayValoriReali(double[] wVals) {
		
		double[] valsReale = new double[wVals.length];
		for(int i=0; i<wVals.length; i++){
			valsReale[i] = funzione.valutaFase(wVals[i]);
		}
		return valsReale;
	}

	/**
	 * riceve il valore X di un punto reale e restituisce la x del punto sul
	 * pannello.
	 * 
	 * @param valX
	 * @return la coordinata x del punto sul pannello oppure -1 se il punto non
	 *         ci sta sul pannello
	 */
	private int trovaX(double valX){
		valX = Math.abs(valX);
		//trovo l'esponente NotazScientifica e poi trovo il logaritmo
		int esp = trovaEsponenteNS(valX);
		double mant = valX*pow(10, -esp);
		if(mant==1 && esp == logwMax){
			return getWidth()-1;
		}
		if(esp<logwMin || esp>logwMax-1){
			return -1;
		}
		double numeroDecadi = logwMax-logwMin;
		double pixPerDecade = (double)getWidth()/numeroDecadi;
		double offset = pixPerDecade * (esp-logwMin);
		double logVal = log10(mant);
		double res = logVal*pixPerDecade + offset;
		return (int) res;
	}

	/**
	 * riceve il valore Y di un punto reale e restituisce la y del punto sul
	 * pannello.
	 * 
	 * @param valY
	 * @return la coordinata y del punto sul pannello oppure -1 se il punto non
	 *         ci sta sul pannello
	 */
	private int trovaY(double valY){
		if(valY<fMin || valY>fMax){
			return -1;
		}
		double v = (1- (valY-fMin)/(fMax-fMin));
		double res = v * getHeight();
		return (int) res;
	}
	
	
	/**
	 * riceve la coordinata x di un punto sul pannello e restituisce il valore
	 * approssimato di w in quel punto
	 * 
	 * @param coordx
	 * @return
	 */
	private double revTrovaX(int coordx){
		//lunghezza di una decade in pixel
		int numeroDecadi = logwMax-logwMin;
		double l = (double)getWidth()/(double)numeroDecadi;
		double a = (coordx % l) / l;
		int b = logwMin + (int) ((double)coordx/l);
		return pow(10, a+b);
	}
	
	
	/**
	 * riceve la coordinata y di un punto sul pannello e restituisce la fase, il
	 * valore della funzione in quel punto
	 * 
	 * @param coordy
	 * @return
	 *
	private double revTrovaY(int coordy){
		double m = 1 - (double)coordy/(double)getHeight();
		return m*(fMax-fMin) + fMin;
	}
	*/
	
	/**
	 * una volta trovata la pulsazione di margine di ampiezza, trova il margine di
	 * ampiezza e setta la label sul pannello
	 */
	private void trovataWma(double w){
		double dbW = funzione.valutaModuloDB(w);
		finestra.setLabelMargineAmpiezza("Margine ampiezza:   Ma = "
				+ formatDouble(-dbW, 3)
				+ " dB    ( Wma = "
				+ formatDouble(w, 3)
				+ " rad/s )");
	}

	/**
	 * Il calcolo della pulsazione di margine di ampiezza (pulsazione alla quale
	 * fase(G(jw))=-180)  approssimato. Ma meglio di cosi non so fare... se
	 * lo vuoi esatto comprati MatLab :P
	 * 
	 * @param xStart
	 * @param xEnd
	 */
	private void calcolaWma(int xStart, int xEnd) {
		if(xEnd-xStart>10 || xStart<0 || xEnd<0){
			return;
		}
		
		//la coordinata dell'asse -180
		int y180 = trovaY(-180);
		
		//trovo wStart e aggiusto 
		double wStart = revTrovaX(xStart);
		double increment = wStart/1000;
		double faseWStart = funzione.valutaFase(wStart);
		while(trovaY(faseWStart)==y180){
			wStart -= increment;
			faseWStart = funzione.valutaFase(wStart);
		}
		
		//trovo wEnd
		double wEnd = revTrovaX(xEnd);
		increment = wEnd/1000;
		double faseWEnd = funzione.valutaFase(wEnd);
		while(trovaY(faseWEnd)==y180){
			wEnd += increment;
			faseWEnd = funzione.valutaFase(wEnd);
		}
		
		//voglio trovare una w tale che minimizzi l'errore |180+f(w)|
		double wErrMin = wStart;
		//cerco i valori incrementando di un decimillesimo
		double delta = (wEnd-wStart)/10000;
		double erroreMinimo = Math.abs(180+funzione.valutaFase(wErrMin));
		for(double i=wStart; i<=wEnd; i+=delta){
			double err = Math.abs(180+funzione.valutaFase(i));
			if(err<erroreMinimo){
				erroreMinimo = err;
				wErrMin=i;
			}
		}
		
		trovataWma(wErrMin);
	}
	

}




























