package gui.pannelliDiag;

import static app.UTIL.ASSE_PRINCIPALE;
import static app.UTIL.ASSE_SECONDARIO;
import static app.UTIL.COLORE_FUNZIONE_ASINT;
import static app.UTIL.COLORE_FUNZIONE_REALE;
import static app.UTIL.LOGWMAX_DEFAULT;
import static app.UTIL.LOGWMIN_DEFAULT;
import static app.UTIL.MASSIMO_M_MIN;
import static app.UTIL.MINIMO_M_MAX;
import static app.UTIL.M_MAX_DEFAULT;
import static app.UTIL.M_MIN_DEFAULT;
import static app.UTIL.PANEL_BACKGR;
import static app.UTIL.formatDouble;
import static app.UTIL.trovaEsponenteNS;
import static java.lang.Math.log10;
import static java.lang.Math.pow;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JPanel;

import app.UTIL;
import fdt.FDT;
import fdt.FattSemplice;
import fdt.Fattore;

@SuppressWarnings("serial")
public class PannelloDiagrammaModuliAsint extends JPanel {

	private int logwMin, logwMax;
	private double mMin, mMax;

	private FDT funzione;
	private PannelloDiagramma finestra;

	public PannelloDiagrammaModuliAsint(FDT funzione, PannelloDiagramma parent) {
		super(null);
		this.funzione = funzione;
		this.finestra = parent;
		setmMin(M_MIN_DEFAULT);
		setmMax(M_MAX_DEFAULT);
		setLogwMin(LOGWMIN_DEFAULT);
		setLogwMax(LOGWMAX_DEFAULT);
		setBackground(PANEL_BACKGR);
	}

	// METODI MUTATORI - ACCESSORI--------------------------------------
	public void setLogwMin(int logwMin) {
		this.logwMin = logwMin;
	}
	public void setLogwMax(int logwMax) {
		this.logwMax = logwMax;
	}
	public void setmMin(double mMin) {
		if (mMin > MASSIMO_M_MIN)
			throw new IllegalArgumentException("deve essere mMin<"
					+ MASSIMO_M_MIN);
		this.mMin = mMin;
	}
	public void setmMax(double mMax) {
		if (mMax < MINIMO_M_MAX)
			throw new IllegalArgumentException("deve essere mMax>"
					+ MINIMO_M_MAX);
		this.mMax = mMax;
	}
	// --------------------------------------------------------------------

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		finestra.azzeraLabelAsseModulo();
		disegnaAsseOrdinate(g);
		disegnaAsseAscisse(g);
		disegnaCornice(g);
		disegnaFunzione(g);
		
	}

	private void disegnaCornice(Graphics g) {
		g.setColor(ASSE_PRINCIPALE);
		g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
	}

	
	private void disegnaAsseAscisse(Graphics g) {
		
		// disegno le linee in corrispondenza delle decadi
		for (int i = logwMin; i <= logwMax; i++) {
			double valX = pow(10, i);
			int cX = trovaX(valX);
			if (cX < 0)
				continue;
			g.setColor(ASSE_PRINCIPALE);
			g.drawLine(cX, 0, cX, getHeight()-1);

			// disegno le lineee in corrisp dei numeretti meno importanti
			for (int j = 2; j <= 9; j++) {
				double valx1 = j * valX;
				cX = trovaX(valx1);
				if (cX < 0)
					continue;
				g.setColor(ASSE_SECONDARIO);
				g.drawLine(cX, 1, cX, getHeight()-2);
			}
		}
		
		// disegno la linea dell'asse
		int cY0 = trovaY(0);
		g.setColor(ASSE_PRINCIPALE);
		g.drawLine(0, cY0, getWidth() - 1, cY0);

	}

	
	private void disegnaAsseOrdinate(Graphics g){
		//distanza in pixel tra le label
		int dp = UTIL.DISTANZA_NUMERETTI_ASSE_ORDINATE;
		//numero di intervallini
		int n = getHeight()/dp;
		//range dei valori disegnabili
		int range = (int) (mMax - mMin);
		//distanza in valore tra una label e l'altra
		int intervallo = range/n;
		//intervallo arrotondato a 5
		int interv = 5 * (intervallo / 5);
		
		g.setColor(ASSE_SECONDARIO);
		
		//aggiungo lo zero
		int y0 = trovaY(0);
		finestra.aggiungiNumerettoAsseModuli(0+"", y0);
		
		//disegno quelli sopra l'asse
		int val = 0;
		
		while(true){
			val += interv;
			int y = trovaY(val);
			if(y==-1) break;
			//disegno l'asse
			g.drawLine(1, y, getWidth()-2, y);
			//aggiungo il numeretto
			finestra.aggiungiNumerettoAsseModuli(val+"", y);
		}
		
		//disegno quelli sotto l'asse
		val = 0;
		while(true){
			val -= interv;
			int y = trovaY(val);
			if(y==-1) break;
			//disegno l'asse
			g.drawLine(1, y, getWidth()-2, y);
			//disegno il numeretto
			finestra.aggiungiNumerettoAsseModuli(val+"", y);
		}
		
	}

	
	private void disegnaFunzione(Graphics g) {
		
		//il valore di w per ogni pixel
		double[] wVals = new double[getWidth()];
		for(int i=0; i<wVals.length; i++){
			wVals[i]=revTrovaX(i);
		}
		
		//il valore della funzione asint in db per ogni w dei pixel
		double[] dbValsAsint = calcolaArrayValoriAsint(wVals);
		
		//il valore della funzione reale in db per ogni w dei pixel
		double[] dbValsReale = calcolaArrayValoriReali(wVals);
		
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		
		if(UTIL.LIVELLO_ASINT_SOTTO){
			//disegno prima il grafico asintotico
			g.setColor(COLORE_FUNZIONE_ASINT);
			if(UTIL.DISEGNA_GRAFICO_ASINTOTICO){
				disegnaGrafico(g, dbValsAsint);
			}
			g.setColor(COLORE_FUNZIONE_REALE);
			if(UTIL.DISEGNA_GRAFICO_REALE){
				disegnaGrafico(g, dbValsReale);
			}
		}else{
			g.setColor(COLORE_FUNZIONE_REALE);
			if(UTIL.DISEGNA_GRAFICO_REALE){
				disegnaGrafico(g, dbValsReale);
			}
			g.setColor(COLORE_FUNZIONE_ASINT);
			if(UTIL.DISEGNA_GRAFICO_ASINTOTICO){
				disegnaGrafico(g, dbValsAsint);
			}
		}
		
		//per il calcolo della pulsazione di attraversamento
		int xStart = -1;
		int xEnd = -1;
		int y0 = trovaY(0);
		for(int i=1; i<wVals.length; i++){
			if(dbValsReale[i]>mMin && dbValsReale[i]<mMax){
				if(trovaY(dbValsReale[i-1])==y0){
					if(xStart==-1) xStart = i-1;
					xEnd = i-1;
				}
			}
		}
		calcoloWt(xStart, xEnd);
		
	}
	
	private void disegnaGrafico(Graphics g, double[] vals){
		int x1 = 0;
		int y1 = trovaY(vals[0]);
		for(int i=1; i<getWidth(); i++){
			int y2 = trovaY(vals[i]);
			if(y2!=y1){
				g.drawLine(x1, y1, i, y2);
				x1=i;
				y1=y2;
			}
		}
		g.drawLine(x1, y1, getWidth()-1, trovaY(vals[getWidth()-1]));
	}

	private double[] calcolaArrayValoriReali(double[] wVals) {
		double[] dbValsReale = new double[wVals.length];
		for(int i=0; i<wVals.length; i++){
			dbValsReale[i] = funzione.valutaModuloDB(wVals[i]);
		}
		return dbValsReale;
	}

	private double[] calcolaArrayValoriAsint(double[] wVals) {
		
		double[] dbVals = new double[wVals.length];
		//devo calcolare a quanti decibel corrisponde un aumento di 20db/decade
		//numero di pixels / numero di decadi = ottengo il numero di pixel per decade
		double numeroDecadi = logwMax - logwMin;
		double pixPerDecade = (double) getWidth() / numeroDecadi;
		//l'aumento  uguale a 20/numpixperdecade
		double aumento20perPix = 20/pixPerDecade;
		
		double guadagnoStaticoInDB = UTIL.DB(funzione.getGuadagnoStatico());
		for(int i=0; i<dbVals.length; i++){
			
			//guadagno statico
			dbVals[i] += guadagnoStaticoInDB;
			
			//poli o zeri nell'origine
			int h = funzione.getZeriPoliOrigine();
			//vale zero alla pulsazione 1
			//trovo il pixel al quale w=1
			int pix0 = trovaX(1);
			
			dbVals[i] += (i-pix0) * aumento20perPix * h;
			
			for(Fattore fattore : funzione.getFattori()){
				if (fattore instanceof FattSemplice) {
					FattSemplice fs = (FattSemplice) fattore;
					
					//mi serve il punto a partire dal quale devo incrementare/decrementare
					//trovo il pixel al quale w=log10(a)
					//ricordati che a  il coefficiente di w; quindi devo fare 1/a
					int pixA = trovaX(1/fs.getA());
					if(i>pixA){
						dbVals[i] += (i-pixA)*aumento20perPix*fs.getM();
					}
				}
			}
			
		}
		return dbVals;
	}

	/**
	 * riceve il valore X di un punto reale e restituisce la x del punto sul
	 * pannello.
	 * 
	 * @param valX
	 * @return la coordinata x del punto sul pannello oppure -1 se il punto non
	 *         ci sta sul pannello
	 */
	private int trovaX(double valX) {
		valX = Math.abs(valX);
		// trovo l'esponente NotazScientifica e poi trovo il logaritmo
		int esp = trovaEsponenteNS(valX);
		double mant = valX * pow(10, -esp);
		if (mant == 1 && esp == logwMax) {
			return getWidth() - 1;
		}
		if (esp < logwMin || esp > logwMax - 1) {
			return -1;
		}
		double numeroDecadi = logwMax - logwMin;
		double pixPerDecade = (double) getWidth() / numeroDecadi;
		double offset = pixPerDecade * (esp - logwMin);
		double logVal = log10(mant);
		double res = logVal * pixPerDecade + offset;
		return (int) res;
	}

	/**
	 * riceve il valore Y di un punto reale e restituisce la y del punto sul
	 * pannello.
	 * 
	 * @param valY
	 * @return la coordinata y del punto sul pannello oppure -1 se il punto non
	 *         ci sta sul pannello
	 */
	private int trovaY(double valY) {
		if (valY < mMin || valY > mMax) {
			return -1;
		}
		double v = (1 - (valY - mMin) / (mMax - mMin));
		double res = v * getHeight();
		return (int) res;
	}

	/**
	 * riceve la coordinata x di un punto sul pannello e restituisce il valore
	 * approssimato di w in quel punto
	 * 
	 * @param coordx
	 * @return
	 */
	private double revTrovaX(int coordx) {
		// lunghezza di una decade in pixel
		int numeroDecadi = logwMax - logwMin;
		double l = (double) getWidth() / (double) numeroDecadi;
		double a = (coordx % l) / l;
		int b = logwMin + (int) ((double) coordx / l);
		return pow(10, a + b);
	}

	/**
	 * riceve la coordinata y di un punto sul pannello e restituisce il modulo,
	 * il valore della funzione in quel punto
	 * 
	 * @param coordy
	 * @return
	 *
	private double revTrovaY(int coordy) {
		double m = 1 - (double) coordy / (double) getHeight();
		return m * (mMax - mMin) + mMin;
	}
	*/
	
	/**
	 * una volta trovata la pulsazione di attraversamento, trova il margine di
	 * fase e setta la label sul pannello
	 */
	private void trovataWt(double w){
		double faseWt = funzione.valutaFase(w);
		finestra.setLabelMargineFase("Margine di fase:    Mf = "
				+ formatDouble(180+faseWt, 3) 
				+ "    ( Wt = "
				+ formatDouble(w, 3)
				+ " rad/s )");
	}
	
	/**
	 * Il calcolo della pulsazione di attraversamento  approssimato. Ma meglio
	 * di cosi non so fare... se lo vuoi esatto comprati MatLab :P
	 * 
	 * @param xStart
	 * @param xEnd
	 */
	private void calcoloWt(int xStart, int xEnd) {
		if(xEnd-xStart>10 || xStart<0 || xEnd<0){
			return;
		}
		
		//la coordinata dell'asse 0
		int y0 = trovaY(0);
		
		//trovo wStart e aggiusto 
		double wStart = revTrovaX(xStart);
		double increment = wStart/1000;
		double moduloWStart = funzione.valutaModuloDB(wStart);
		while(trovaY(moduloWStart)==y0){
			wStart -= increment;
			moduloWStart = funzione.valutaModuloDB(wStart);
		}
		
		//trovo wEnd
		double wEnd = revTrovaX(xEnd);
		increment = wEnd/1000;
		double moduloWEnd = funzione.valutaModuloDB(wEnd);
		while(trovaY(moduloWEnd)==y0){
			wEnd += increment;
			moduloWEnd = funzione.valutaModuloDB(wEnd);
		}
		
		//voglio trovare una w tale che minimizzi l'errore |m(w)|
		double wErrMin = wStart;
		//cerco i valori incrementando di un decimillesimo
		double delta = (wEnd-wStart)/10000;
		double erroreMinimo = Math.abs(funzione.valutaModuloDB(wErrMin));
		for(double i=wStart; i<=wEnd; i+=delta){
			double err = Math.abs(funzione.valutaModuloDB(i));
			if(err<erroreMinimo){
				erroreMinimo = err;
				wErrMin=i;
			}
		}
		
		trovataWt(wErrMin);
	}

}












