package gui.pannelliGestione;

import gui.pannelliGestione.graphicsFdtElem.GraphicElement;
import gui.pannelliGestione.graphicsFdtElem.GraphicFCC;
import gui.pannelliGestione.graphicsFdtElem.GraphicFS;
import gui.pannelliGestione.graphicsFdtElem.GraphicFattore;
import gui.pannelliGestione.graphicsFdtElem.GraphicFunctionDef;
import gui.pannelliGestione.graphicsFdtElem.GraphicH;
import gui.pannelliGestione.graphicsFdtElem.GraphicK;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

@SuppressWarnings("serial")
public class CanvasFunzione extends JPanel {
	
	private GraphicFunctionDef funcDef;
	private GraphicK k;
	private GraphicH h;
	private List<GraphicFattore> fattori;
	private EditorTextPanel editorTextPanel;
	
	private final Rectangle selezione;
	
	private static final int interspace = 3;
	
	public CanvasFunzione(EditorTextPanel editorTextPanel){
		this.editorTextPanel=editorTextPanel;
		setBackground(Color.white);
		funcDef = new GraphicFunctionDef();
		k = new GraphicK();
		h = new GraphicH();
		fattori = new LinkedList<GraphicFattore>();
		selezione = new Rectangle();
		
		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseMoved(MouseEvent e) {
				CanvasFunzione.this.mouseMoved(e);
			}
		});
		
		addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				CanvasFunzione.this.mouseClicked(e);
			}
		});
		
	}
	
	private void mouseMoved(MouseEvent e) {
		if(k.getUltimaPosizione().contains(e.getPoint())){
			selezione.setBounds(k.getUltimaPosizione());
			repaint();
			return;
		}
		if(h.getUltimaPosizione().contains(e.getPoint())){
			selezione.setBounds(h.getUltimaPosizione());
			repaint();
			return;
		}
		for(GraphicElement elem : fattori){
			if(elem.getUltimaPosizione().contains(e.getPoint())){
				selezione.setBounds(elem.getUltimaPosizione());
				repaint();
				return;
			}
		}
		selezione.setBounds(0, 0, 0, 0);
		repaint();
	}
	
	protected void mouseClicked(MouseEvent e) {
		selezione.setBounds(0, 0, 0, 0);
		if(k.getUltimaPosizione().contains(e.getPoint())){
			k.setK(EditorGraficoDialogs.showModificaK(getParent(), k.getK()));
			repaint();
			return;
		}
		if(h.getUltimaPosizione().contains(e.getPoint())){
			h.setMolteplicita(EditorGraficoDialogs.showModificaH(getParent(), h.getMolteplicita()));
			repaint();
			return;
		}
		for(GraphicFattore fattore : fattori){
			if(fattore.getUltimaPosizione().contains(e.getPoint())){
				if(e.getButton()==3){
					if(JOptionPane.showConfirmDialog(null, "Eliminare il fattore selezionato?")
							== JOptionPane.YES_OPTION){
						fattori.remove(fattore);
						repaint();
					}
				}else{
					EditorGraficoDialogs.showModificaFattore(getParent(), fattore);
					repaint();
				}
				return;
			}
		}
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		aggiornaEditorTesto();
		
		Graphics2D g2 = (Graphics2D)g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
		
		int supMargin = 10;
		int leftMargin = 10;
		
		int hFunzione = 80;
		
		
		int x = leftMargin;
		final int middleY = supMargin + hFunzione/2;
		
		//varie evidenziazioni in fase di test
//		g.setColor(Color.yellow);
//		g.fillRect(x, supMargin, getWidth()-2*leftMargin, hFunzione);
//		g.setColor(Color.gray);
//		g.drawLine(x, middleY, getWidth()-leftMargin, middleY);
		
		g.setColor(new Color(0, 0, 255, 60));
		g2.fill(selezione);
		
		g.setColor(Color.black);
		
		//G(s) = 
		int yFunDef = middleY - funcDef.getH()/2;
		funcDef.paintElement(g2, x, yFunDef);
		x += funcDef.getW() + interspace;
		
		//disegna K
		int yK = middleY - k.getH()/2;
		k.paintElement(g2, x, yK);
		x += k.getW() + interspace;
		
		//il punto di moltiplicazione tra k ed s^h
		g.fillOval(x, middleY-2, 4, 4);
		x += 4+interspace;
		
		//disegna s^h
		int yH = middleY - h.getH()/2;
		h.paintElement(g2, x, yH);
		x += h.getW() + interspace;
		
		//il punto di moltiplicazione tra s^h e la frazione
		if(fattori.size()==0) return;
		
		g.fillOval(x, middleY-2, 4, 4);
		x += 4+interspace;
		
		//servirebbe per la dimensione automatica del pannello
		//ma non riesco a farlo... sono stanco xD
		//entra in un ciclo infinito se faccio setSize() qua dentro
		//int panelMinW;
		
		if(lineaDiFrazionePresente()){
			int wTotNum = 0;
			int wTotDen = 0;
			int hMaxNum = 0;
			int hMaxDen = 0;
			
			List<GraphicFattore> listNum = new LinkedList<GraphicFattore>();
			List<GraphicFattore> listDen = new LinkedList<GraphicFattore>();
			
			for(GraphicFattore fattore : fattori){
				if(fattore.getM()>0){
					listNum.add(fattore);
					hMaxNum = Math.max(hMaxNum, fattore.getH());
					wTotNum += fattore.getW();
				}else if(fattore.getM()<0){
					listDen.add(fattore);
					hMaxDen = Math.max(hMaxDen, fattore.getH());
					wTotDen += fattore.getW();
				}
			}
			
			//aggiungo lo spazio occupato dai pallini della moltiplicazione
			if(listNum.size()>0) 
				wTotNum += (listNum.size()-1)*(2*interspace+4);
			if(listDen.size()>0)
				wTotDen += (listDen.size()-1)*(2*interspace+4);
			
			int fractW = Math.max(wTotNum, wTotDen);
			int xStartNum = x+(fractW-wTotNum)/2;
			int xStartDen = x+(fractW-wTotDen)/2;
			
			//disegna la linea di frazione
			g.setColor(Color.black);
			g.drawLine(x, middleY, x+fractW, middleY);
			
			//disegna il numeratore
			if(listNum.size()==0){
				//non ci sono fattori nel numeratore
				wTotNum -= GraphicElement.getStringW("1", GraphicElement.MAIN_FONT)/2;
				g.setFont(GraphicElement.MAIN_FONT);
				g.setColor(Color.black);
				int x1 = xStartNum - GraphicElement.getStringW("1", GraphicElement.MAIN_FONT)/2;
				g.drawString("1", x1, middleY-2);
			}else{
				disegnaFattori(listNum, g2, middleY - hMaxNum/2, xStartNum);
			}
			
			//disegna il denominatore
			disegnaFattori(listDen, g2, middleY + hMaxDen/2, xStartDen);
			
			//panelMinW =  x + fractW + 80;
			
		}else{
			disegnaFattori(fattori, g2, middleY, x);
//			int wFattori = disegnaFattori(fattori, g2, middleY, x);
//			panelMinW =  x + wFattori + 80;
			
		}
		
	}
	
//	private void s(Object x) {
//		System.out.println(x);
//	}

	/**
	 * ritorna la lunghezza totale della stringa di fattori che ha disegnato
	 */
	private int disegnaFattori(List<GraphicFattore> daDisegnare, Graphics2D g, int middleY, int xStart){
		boolean first = true;
		int xInc = 0;
		for(GraphicFattore fattore : daDisegnare){
			g.setColor(Color.black);
			if(first){
				first=false;
			}else{
				g.fillOval(xStart+xInc, middleY-2, 4, 4);
				xInc += 4+interspace;
			}
			fattore.paintElement((Graphics2D)g, xStart+xInc, middleY-fattore.getH()/2);
			xInc += fattore.getW() + interspace;
		}
		return xInc;
	}

	private boolean lineaDiFrazionePresente(){
		for(GraphicFattore i : fattori){
			if(i.getM()<0) return true;
		}
		return false;
	}
	
	
	public void nuovoFattoreSemplice() {
		GraphicFS fs = EditorGraficoDialogs.showNuovoFS(getParent());
		if(fs!=null){
			fattori.add(fs);
			repaint();
		}
	}

	public void nuovoFattoreComplesso() {
		GraphicFCC fcc = EditorGraficoDialogs.showNuovoFCC(getParent());
		if(fcc!=null){
			fattori.add(fcc);
			repaint();
		}
	}

	public void reset() {
		int c = JOptionPane.showConfirmDialog(getParent(), 
				"Azzerare la funzione di trasferimento?");
		if(c==JOptionPane.OK_OPTION){
			k.setK(1);
			h.setMolteplicita(0);
			fattori.clear();
			selezione.setBounds(0, 0, 0, 0);
			repaint();
		}
	}
	
	public String generaTextFDT(){
		StringBuilder sb = new StringBuilder();
		
		sb.append("k = ");
		sb.append(k.getK());
		sb.append("\n");
		
		sb.append("h = ");
		sb.append(h.getMolteplicita());
		sb.append("\n");
		
		for(GraphicFattore f : fattori){
			if (f instanceof GraphicFS) {
				GraphicFS fs = (GraphicFS) f;
				sb.append("FS, a = ");
				sb.append(fs.getTextA());
				sb.append(", m = ");
				sb.append(fs.getM());
				sb.append("\n");
			}else if (f instanceof GraphicFCC) {
				GraphicFCC fcc = (GraphicFCC) f;
				sb.append("FCC, a = ");
				sb.append(fcc.getTextA());
				sb.append(", b = ");
				sb.append(fcc.getTextB());
				sb.append(", m = ");
				sb.append(fcc.getM());
				sb.append("\n");
			}
		}
		return sb.toString();
	}
	
	private void aggiornaEditorTesto(){
		editorTextPanel.setText(generaTextFDT());
	}
	
	

}
