package gui.pannelliGestione;

import java.awt.Font;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.PrintWriter;
import java.util.Scanner;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import app.UTIL;

/**
 * Pannello che fa parte dell'interfaccia grafica.
 * 
 * @author Andrea De Meco
 * 
 */
@SuppressWarnings("serial")
public class EditorTextPanel extends JPanel {

	private File fileCorrente;
	private boolean fileCorrenteModificato;
	private JFileChooser fc;

	private JLabel l;
	private JToolBar tb;
	private JTextArea ta;
	private JScrollPane sp;
	
	private boolean beta = true;

	public EditorTextPanel() {
		super(null);
		
		setBorder(BorderFactory.createTitledBorder("Editor testuale"));
		
		fc = new JFileChooser();
		fc.setFileFilter(new FileNameExtensionFilter("File di testo (*.txt)", "txt"));
		fc.setMultiSelectionEnabled(false);
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);

		l = new JLabel();
		aggiornaLabelFileCorrente();
		createToolBar();
		createTextArea();
		
		add(sp);
		add(tb);

		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				posizionaComponenti();
			}
		});
		
	}

	private void posizionaComponenti() {
		
		int supMargin = 15;
		int infMargin = 10;
		int orizMargin = 15;
		int interlinea = 0;
		
		int availW = getWidth() - orizMargin*2;
		int availH = getHeight() - supMargin - infMargin;
		
		int hToolBar = tb.getPreferredSize().height;
		int hTA = availH - (hToolBar + interlinea);
		
		int yToolBar = supMargin;
		int yTA = yToolBar + hToolBar + interlinea;
		
		tb.setBounds(orizMargin, yToolBar, availW, hToolBar);
		sp.setBounds(orizMargin, yTA, availW, hTA);
		sp.validate();
		
	}

	private void createToolBar() {
		
		final Action aSave = new AbstractAction("", UTIL.getIcon("save.gif", -1, 18)) {
			public void actionPerformed(ActionEvent e) {
				if(beta){
					usareEditorGrafico();
					return;
				}
				salva();
			}
		};

		Action aNew = new AbstractAction("", UTIL.getIcon("new.gif", -1, 18)) {
			public void actionPerformed(ActionEvent e) {
				if(beta){
					usareEditorGrafico();
					return;
				}
				if(!chiudiFile()) return;
				ta.setText("");
				fileCorrente = null;
				fileCorrenteModificato = false;
				aggiornaLabelFileCorrente();
			}
		};

		Action aOpen = new AbstractAction("", UTIL.getIcon("open.gif", -1, 18)) {
			public void actionPerformed(ActionEvent e) {
				if(beta){
					usareEditorGrafico();
					return;
				}
				
				if(!chiudiFile()) return;
				
				if (fc.showOpenDialog(EditorTextPanel.this) == JFileChooser.APPROVE_OPTION) {
					fileCorrente = fc.getSelectedFile();
				} else {
					return;
				}
				try {
					Scanner sc = new Scanner(fileCorrente);
					ta.setText("");
					while (sc.hasNextLine()) {
						ta.append(sc.nextLine() + "\n");
					}
					sc.close();
					fileCorrenteModificato = false;
					aggiornaLabelFileCorrente();
				} catch (Exception exc) {
					UTIL.dialogoException("Si  verificato un errore.", exc);
				}
			}
		};

		JButton b1 = new JButton(aNew);
		b1.setToolTipText("Nuovo...");
		b1.setFocusable(false);
		JButton b2 = new JButton(aOpen);
		b2.setToolTipText("Apri...");
		b2.setFocusable(false);
		JButton b3 = new JButton(aSave);
		b3.setToolTipText("Salva...");
		b3.setFocusable(false);

		tb = new JToolBar();
		tb.addSeparator();
		tb.add(b1);
		tb.add(b2);
		tb.add(b3);
		tb.add(l);
		tb.setFloatable(false);
	}

	private void createTextArea() {
		ta = new JTextArea();
		sp = new JScrollPane(ta);
		ta.setFont(new Font("Lucida console", 0, 14));
		ta.setMargin(new Insets(5, 5, 5, 5));
		ta.setEnabled(false);
		ta.setFocusable(false);
		ta.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				if(beta){
					usareEditorGrafico();
					return;
				}
				if (e.getKeyChar() != KeyEvent.CHAR_UNDEFINED) {
					fileCorrenteModificato = true;
					aggiornaLabelFileCorrente();
				}
			}
		});
		ta.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				if(beta){
					usareEditorGrafico();
					return;
				}
			}
		});
	}

	private void usareEditorGrafico() {
		JOptionPane.showMessageDialog(EditorTextPanel.this, 
				"Questa  una versione beta. Per il momento l'editing " +
				"testuale  disabilitato.\nUtilizzare l'editor grafico.");
	}

	private void aggiornaLabelFileCorrente() {

		String file;
		if (fileCorrente == null) {
			if (fileCorrenteModificato) {
				file = "[nuovo]";
			} else {
				file = "[nessuno]";
			}
		} else {
			file = fileCorrente.getName()
					+ (fileCorrenteModificato ? " [modificato]" : "");
		}
		l.setText(" File correntemente aperto: " + file);

		if (fileCorrente != null)
			l.setToolTipText("" + fileCorrente);
	}

	public File getFileAperto() {
		return fileCorrente;
	}
	
	/**
	 * @return	true se il testo dell'editor non  stato salvato.
	 */
	public boolean contieneModifiche() {
		return fileCorrenteModificato;
	}

	public String getText() {
		return ta.getText();
	}
	
	public void setText(String text){
		ta.setText(text);
		fileCorrenteModificato=true;
	}

	/**
	 * se il file correntemente aperto  modificato chiede all'utente se vuole
	 * salvare il file.
	 * 
	 * @return ritorna true se l'utente ha salvato oppure non vuole salvare il
	 *         file. ritorna false se l'utente ha annullato la chiusura, oppure
	 *         il salvataggio non  andato a buon fine
	 */
	public boolean chiudiFile() {
		if (fileCorrenteModificato) {
			int x = conf("Il file corrente  modificato. Salvare le modifiche?");
			if (x == JOptionPane.YES_OPTION) {
				return salva();
			} else {
				if (x == JOptionPane.CANCEL_OPTION) {
					return false;
				}else{
					return true;
				}
			}
		}else{
			return true;
		}
	}
	
	/**
	 * salva il testo corrente su un file. Se un file  correntemente aperto
	 * allora salva su questo file. Se nessun file  aperto, fa scegliere
	 * all'utente il file su cui salvare.
	 * 
	 * @return <b>true</b> se il salvataggio  andato a buon fine.<br>
	 *         <b>false</b> se il file non  stato salvato (per un errore o per
	 *         scelta dell'utente).
	 */
	private boolean salva(){
		
		if (fileCorrente == null) {
			if (fc.showSaveDialog(EditorTextPanel.this) == JFileChooser.APPROVE_OPTION) {
				fileCorrente = fc.getSelectedFile();
				FileFilter ff = fc.getFileFilter();
				if(ff instanceof FileNameExtensionFilter){
					FileNameExtensionFilter fnef = (FileNameExtensionFilter)ff;
					if(fnef.getExtensions().length>0){
						String ext = fnef.getExtensions()[0];
						String fileName = fileCorrente.getAbsolutePath();
						if(!fileName.endsWith("."+ext)){
							fileCorrente = new File(fileName.concat("."+ext));
						}
					}
				}
			} else {
				return false;
			}
		}
		
		try {
			PrintWriter pw = new PrintWriter(fileCorrente);
			pw.println(ta.getText());
			pw.close();
			fileCorrenteModificato = false;
			aggiornaLabelFileCorrente();
			return true;
		} catch (Exception exc) {
			UTIL.dialogoException("Si  verificato un errore.", exc);
			return false;
		}
		
	}

	private int conf(String s) {
		return JOptionPane.showConfirmDialog(this, s);
	}
	
}
