package gui.pannelliGestione.graphicsFdtElem;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.LineMetrics;

public abstract class GraphicElement {
	
	private static final String FONT_NAME = ""; //alternativamente cambria math
	public static final Font MAIN_FONT = new Font(FONT_NAME, Font.PLAIN, 18);
	public static final Font BIG_FONT = new Font(FONT_NAME, Font.PLAIN, 36);
	public static final Font LITTLE_FONT = new Font(FONT_NAME, Font.PLAIN, 12);
	
	protected static final FontRenderContext frc = new FontRenderContext(null, true, false);
	
	public static final LineMetrics MAIN_FONT_METRICS = MAIN_FONT.getLineMetrics("s", frc);
	public static final LineMetrics BIG_FONT_METRICS = BIG_FONT.getLineMetrics("s", frc);
	public static final LineMetrics LITTLE_FONT_METRICS = LITTLE_FONT.getLineMetrics("s", frc);
	
	public static final int MAIN_FONT_HEIGHT = (int)MAIN_FONT_METRICS.getHeight();
	public static final int BIG_FONT_HEIGHT = (int)BIG_FONT_METRICS.getHeight();
	public static final int LITTLE_FONT_HEIGHT = (int)LITTLE_FONT_METRICS.getHeight();
	
	public static final int MAIN_FONT_BASELINE = (int)(MAIN_FONT_HEIGHT - MAIN_FONT_METRICS.getDescent());
	public static final int BIG_FONT_BASELINE = (int)(BIG_FONT_HEIGHT - BIG_FONT_METRICS.getDescent());
	public static final int LITTLE_FONT_BASELINE = (int)(LITTLE_FONT_HEIGHT - LITTLE_FONT_METRICS.getDescent());
	
	protected Rectangle ultimaPosizione;
	
	public static int getStringW(String x, Font f){
		return (int) f.getStringBounds(x,frc).getWidth();
	}
	
	public GraphicElement() {
		ultimaPosizione = new Rectangle();
	}
	
	/**
	 * disegna l'elemento sul graphics passato come argomento
	 * 
	 * @param g
	 *            il graphics su cui disegnare
	 * @param x
	 *            la coordinata x del punto top-left
	 * @param y
	 *            la coordinata y del punto top-left
	 */
	public abstract void paintElement(Graphics2D g, int x, int y);
	
	/**
	 * 
	 * @return la larghezza dell'elemento grafico
	 */
	public int getW(){
		return ultimaPosizione.width;
	}
	
	/**
	 * 
	 * @return l'altezza dell'elemento grafico
	 */
	public int getH(){
		return ultimaPosizione.height;
	}
	
	/**
	 * 
	 * @return l'ultima rettangolo di visualizzazione in cui  stato disegnato
	 *         questo elemento
	 */
	public Rectangle getUltimaPosizione() {
		return ultimaPosizione;
	}
	
	/**
	 * ritorna il valore assoluto (il numero senza segno) del double formattato, 
	 * 	precisione massima: 6 decimali
	 * @param num
	 * @return 
	 */
	public static String formatDouble(double num){
		String s1 = ""+num;
		if(s1.startsWith("-")){
			s1 = s1.substring(1);
		}
		int index = s1.indexOf(".");
		if(index==-1){
			return s1;
		}
		int count = 0; //caratteri da rimuovere
		for(int i=s1.length()-1; i>index; i--){
			if(s1.charAt(i)=='0'){
				count++;
			}else{
				break;
			}
		}
		if(s1.length()-count-1==index){
			//elimino anche il punto
			count++;
		}
		return s1.substring(0, s1.length()-count);
	}

}
